@@ -1,20 +1,11 @@
-#include <string.h>
 #include <Python.h>
 #include <sass/context.h>
 
 #if PY_MAJOR_VERSION >= 3
 #define PySass_IF_PY3(three, two) (three)
-#define PySass_Int_FromLong(v) PyLong_FromLong(v)
-#define PySass_Bytes_Check(o) PyBytes_Check(o)
-#define PySass_Bytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
-#define PySass_Bytes_AS_STRING(o) PyBytes_AS_STRING(o)
 #define PySass_Object_Bytes(o) PyUnicode_AsUTF8String(PyObject_Str(o))
 #else
 #define PySass_IF_PY3(three, two) (two)
-#define PySass_Int_FromLong(v) PyInt_FromLong(v)
-#define PySass_Bytes_Check(o) PyString_Check(o)
-#define PySass_Bytes_GET_SIZE(o) PyString_GET_SIZE(o)
-#define PySass_Bytes_AS_STRING(o) PyString_AS_STRING(o)
 #define PySass_Object_Bytes(o) PyObject_Str(o)
 #endif
 
@@ -77,6 +68,8 @@ static PyObject* _to_py_value(const union Sass_Value* value) {
             size_t i = 0;
             PyObject* items = PyTuple_New(sass_list_get_length(value));
             PyObject* separator = sass_comma;
+            int is_bracketed = sass_list_get_is_bracketed(value);
+            PyObject* bracketed = PyBool_FromLong(is_bracketed);
             switch (sass_list_get_separator(value)) {
                 case SASS_COMMA:
                     separator = sass_comma;
@@ -96,7 +89,7 @@ static PyObject* _to_py_value(const union Sass_Value* value) {
                 );
             }
             retv = PyObject_CallMethod(
-                types_mod, "SassList", "OO", items, separator
+                types_mod, "SassList", "OOO", items, separator, bracketed
             );
             break;
         }
@@ -160,6 +153,7 @@ static union Sass_Value* _list_to_sass_value(PyObject* value) {
     Py_ssize_t i = 0;
     PyObject* items = PyObject_GetAttrString(value, "items");
     PyObject* separator = PyObject_GetAttrString(value, "separator");
+    PyObject* bracketed = PyObject_GetAttrString(value, "bracketed");
     Sass_Separator sep = SASS_COMMA;
     if (separator == sass_comma) {
         sep = SASS_COMMA;
@@ -168,7 +162,8 @@ static union Sass_Value* _list_to_sass_value(PyObject* value) {
     } else {
         assert(0);
     }
-    retv = sass_make_list(PyTuple_Size(items), sep);
+    int is_bracketed = bracketed == Py_True;
+    retv = sass_make_list(PyTuple_Size(items), sep, is_bracketed);
     for (i = 0; i < PyTuple_Size(items); i += 1) {
         sass_list_set_value(
             retv, i, _to_sass_value(PyTuple_GET_ITEM(items, i))
@@ -179,6 +174,7 @@ static union Sass_Value* _list_to_sass_value(PyObject* value) {
     Py_DECREF(sass_space);
     Py_DECREF(items);
     Py_DECREF(separator);
+    Py_DECREF(bracketed);
     return retv;
 }
 
@@ -206,7 +202,7 @@ static union Sass_Value* _number_to_sass_value(PyObject* value) {
     PyObject* unit = PyObject_GetAttrString(value, "unit");
     PyObject* bytes = PyUnicode_AsEncodedString(unit, "UTF-8", "strict");
     retv = sass_make_number(
-        PyFloat_AsDouble(d_value), PySass_Bytes_AS_STRING(bytes)
+        PyFloat_AsDouble(d_value), PyBytes_AS_STRING(bytes)
     );
     Py_DECREF(d_value);
     Py_DECREF(unit);
@@ -217,7 +213,7 @@ static union Sass_Value* _number_to_sass_value(PyObject* value) {
 static union Sass_Value* _unicode_to_sass_value(PyObject* value) {
     union Sass_Value* retv = NULL;
     PyObject* bytes = PyUnicode_AsEncodedString(value, "UTF-8", "strict");
-    retv = sass_make_string(PySass_Bytes_AS_STRING(bytes));
+    retv = sass_make_string(PyBytes_AS_STRING(bytes));
     Py_DECREF(bytes);
     return retv;
 }
@@ -226,7 +222,7 @@ static union Sass_Value* _warning_to_sass_value(PyObject* value) {
     union Sass_Value* retv = NULL;
     PyObject* msg = PyObject_GetAttrString(value, "msg");
     PyObject* bytes = PyUnicode_AsEncodedString(msg, "UTF-8", "strict");
-    retv = sass_make_warning(PySass_Bytes_AS_STRING(bytes));
+    retv = sass_make_warning(PyBytes_AS_STRING(bytes));
     Py_DECREF(msg);
     Py_DECREF(bytes);
     return retv;
@@ -236,7 +232,7 @@ static union Sass_Value* _error_to_sass_value(PyObject* value) {
     union Sass_Value* retv = NULL;
     PyObject* msg = PyObject_GetAttrString(value, "msg");
     PyObject* bytes = PyUnicode_AsEncodedString(msg, "UTF-8", "strict");
-    retv = sass_make_error(PySass_Bytes_AS_STRING(bytes));
+    retv = sass_make_error(PyBytes_AS_STRING(bytes));
     Py_DECREF(msg);
     Py_DECREF(bytes);
     return retv;
@@ -265,7 +261,7 @@ static union Sass_Value* _unknown_type_to_sass_error(PyObject* value) {
         format_meth, type_name, NULL
     );
     PyObject* bytes = PyUnicode_AsEncodedString(result, "UTF-8", "strict");
-    retv = sass_make_error(PySass_Bytes_AS_STRING(bytes));
+    retv = sass_make_error(PyBytes_AS_STRING(bytes));
     Py_DECREF(type);
     Py_DECREF(type_name);
     Py_DECREF(fmt);
@@ -304,7 +300,7 @@ static PyObject* _exception_to_bytes() {
 
 static union Sass_Value* _exception_to_sass_error() {
     PyObject* bytes = _exception_to_bytes();
-    union Sass_Value* retv = sass_make_error(PySass_Bytes_AS_STRING(bytes));
+    union Sass_Value* retv = sass_make_error(PyBytes_AS_STRING(bytes));
     Py_DECREF(bytes);
     return retv;
 }
@@ -313,7 +309,7 @@ static Sass_Import_List _exception_to_sass_import_error(const char* path) {
     PyObject* bytes = _exception_to_bytes();
     Sass_Import_List import_list = sass_make_import_list(1);
     import_list[0] = sass_make_import_entry(path, 0, 0);
-    sass_import_set_error(import_list[0], PySass_Bytes_AS_STRING(bytes), 0, 0);
+    sass_import_set_error(import_list[0], PyBytes_AS_STRING(bytes), 0, 0);
     Py_DECREF(bytes);
     return import_list;
 }
@@ -335,8 +331,8 @@ static union Sass_Value* _to_sass_value(PyObject* value) {
         retv = sass_make_boolean(value == Py_True);
     } else if (PyUnicode_Check(value)) {
         retv = _unicode_to_sass_value(value);
-    } else if (PySass_Bytes_Check(value)) {
-        retv = sass_make_string(PySass_Bytes_AS_STRING(value));
+    } else if (PyBytes_Check(value)) {
+        retv = sass_make_string(PyBytes_AS_STRING(value));
     /* XXX: PyMapping_Check returns true for lists and tuples in python3 :( */
     /* XXX: pypy derps on dicts: https://bitbucket.org/pypy/pypy/issue/1970 */
     } else if (PyDict_Check(value) || PyObject_IsInstance(value, mapping_t)) {
@@ -410,7 +406,7 @@ static void _add_custom_functions(
         PyObject* sass_function = PyList_GET_ITEM(custom_functions, i);
         PyObject* signature = PySass_Object_Bytes(sass_function);
         Sass_Function_Entry fn = sass_make_function(
-            PySass_Bytes_AS_STRING(signature),
+            PyBytes_AS_STRING(signature),
             _call_py_f,
             sass_function
         );
@@ -464,8 +460,8 @@ static Sass_Import_List _call_py_importer_f(
         /* We need to give copies of these arguments; libsass handles
          * deallocation of them later, whereas path_str is left flapping
          * in the breeze -- it's treated const, so that's okay. */
-        if (source_str) source_str = strdup(source_str);
-        if (sourcemap_str) sourcemap_str = strdup(sourcemap_str);
+        if (source_str) source_str = sass_copy_c_string(source_str);
+        if (sourcemap_str) sourcemap_str = sass_copy_c_string(sourcemap_str);
 
         sass_imports[i] = sass_make_import_entry(
             path_str, source_str, sourcemap_str
@@ -509,6 +505,17 @@ static void _add_custom_importers(
     sass_option_set_c_importers(options, importer_list);
 }
 
+static void _add_custom_import_extensions(
+        struct Sass_Options* options, PyObject* custom_import_extensions
+) {
+    Py_ssize_t i;
+
+    for (i = 0; i < PyList_GET_SIZE(custom_import_extensions); i += 1) {
+        PyObject* ext = PyList_GET_ITEM(custom_import_extensions, i);
+        sass_option_push_import_extension(options, PyBytes_AS_STRING(ext));
+    }
+}
+
 static PyObject *
 PySass_compile_string(PyObject *self, PyObject *args) {
     struct Sass_Context *ctx;
@@ -520,17 +527,19 @@ PySass_compile_string(PyObject *self, PyObject *args) {
     int source_comments, error_status, precision, indented;
     PyObject *custom_functions;
     PyObject *custom_importers;
+    PyObject *custom_import_extensions;
     PyObject *result;
 
     if (!PyArg_ParseTuple(args,
-                          PySass_IF_PY3("yiiyiOiO", "siisiOiO"),
+                          PySass_IF_PY3("yiiyiOiOO", "siisiOiOO"),
                           &string, &output_style, &source_comments,
                           &include_paths, &precision,
-                          &custom_functions, &indented, &custom_importers)) {
+                          &custom_functions, &indented, &custom_importers,
+                          &custom_import_extensions)) {
         return NULL;
     }
 
-    context = sass_make_data_context(strdup(string));
+    context = sass_make_data_context(sass_copy_c_string(string));
     options = sass_data_context_get_options(context);
     sass_option_set_output_style(options, output_style);
     sass_option_set_source_comments(options, source_comments);
@@ -539,6 +548,7 @@ PySass_compile_string(PyObject *self, PyObject *args) {
     sass_option_set_is_indented_syntax_src(options, indented);
     _add_custom_functions(options, custom_functions);
     _add_custom_importers(options, custom_importers);
+    _add_custom_import_extensions(options, custom_import_extensions);
     sass_compile_data_context(context);
 
     ctx = sass_data_context_get_context(context);
@@ -564,30 +574,33 @@ PySass_compile_filename(PyObject *self, PyObject *args) {
     Sass_Output_Style output_style;
     int source_comments, error_status, precision;
     PyObject *source_map_filename, *custom_functions, *custom_importers,
-             *result;
+             *result, *output_filename_hint, *custom_import_extensions;
 
     if (!PyArg_ParseTuple(args,
-                          PySass_IF_PY3("yiiyiOOO", "siisiOOO"),
+                          PySass_IF_PY3("yiiyiOOOOO", "siisiOOOOO"),
                           &filename, &output_style, &source_comments,
                           &include_paths, &precision,
                           &source_map_filename, &custom_functions,
-                          &custom_importers)) {
+                          &custom_importers, &output_filename_hint,
+                          &custom_import_extensions)) {
         return NULL;
     }
 
     context = sass_make_file_context(filename);
     options = sass_file_context_get_options(context);
 
-    if (PySass_Bytes_Check(source_map_filename)) {
-        size_t source_map_file_len = PySass_Bytes_GET_SIZE(source_map_filename);
-        if (source_map_file_len) {
-            char *source_map_file = (char *) malloc(source_map_file_len + 1);
-            strncpy(
-                source_map_file,
-                PySass_Bytes_AS_STRING(source_map_filename),
-                source_map_file_len + 1
+    if (PyBytes_Check(source_map_filename)) {
+        if (PyBytes_GET_SIZE(source_map_filename)) {
+            sass_option_set_source_map_file(
+                options, PyBytes_AS_STRING(source_map_filename)
+            );
+        }
+    }
+    if (PyBytes_Check(output_filename_hint)) {
+        if (PyBytes_GET_SIZE(output_filename_hint)) {
+            sass_option_set_output_path(
+                options, PyBytes_AS_STRING(output_filename_hint)
             );
-            sass_option_set_source_map_file(options, source_map_file);
         }
     }
     sass_option_set_output_style(options, output_style);
@@ -596,6 +609,7 @@ PySass_compile_filename(PyObject *self, PyObject *args) {
     sass_option_set_precision(options, precision);
     _add_custom_functions(options, custom_functions);
     _add_custom_importers(options, custom_importers);
+    _add_custom_import_extensions(options, custom_import_extensions);
     sass_compile_file_context(context);
 
     ctx = sass_file_context_get_context(context);
@@ -615,9 +629,9 @@ PySass_compile_filename(PyObject *self, PyObject *args) {
 
 static PyMethodDef PySass_methods[] = {
     {"compile_string", PySass_compile_string, METH_VARARGS,
-     "Compile a SASS string."},
+     "Compile a Sass string."},
     {"compile_filename", PySass_compile_filename, METH_VARARGS,
-     "Compile a SASS file."},
+     "Compile a Sass file."},
     {NULL, NULL, 0, NULL}
 };
 
@@ -625,10 +639,10 @@ static char PySass_doc[] = "The thin binding of libsass for Python.";
 
 PyObject* PySass_make_enum_dict() {
     PyObject* dct = PyDict_New();
-    PyDict_SetItemString(dct, "nested", PySass_Int_FromLong(SASS_STYLE_NESTED));
-    PyDict_SetItemString(dct, "expanded", PySass_Int_FromLong(SASS_STYLE_EXPANDED));
-    PyDict_SetItemString(dct, "compact", PySass_Int_FromLong(SASS_STYLE_COMPACT));
-    PyDict_SetItemString(dct, "compressed", PySass_Int_FromLong(SASS_STYLE_COMPRESSED));
+    PyDict_SetItemString(dct, "nested", PyLong_FromLong(SASS_STYLE_NESTED));
+    PyDict_SetItemString(dct, "expanded", PyLong_FromLong(SASS_STYLE_EXPANDED));
+    PyDict_SetItemString(dct, "compact", PyLong_FromLong(SASS_STYLE_COMPACT));
+    PyDict_SetItemString(dct, "compressed", PyLong_FromLong(SASS_STYLE_COMPRESSED));
     return dct;
 }
 
