@@ -31,15 +31,15 @@ __all__ = (
     'SassError', 'SassFunction', 'SassList', 'SassMap', 'SassNumber',
     'SassWarning', 'and_join', 'compile', 'libsass_version',
 )
-__version__ = '0.12.3'
+__version__ = '0.14.5'
 libsass_version = _sass.libsass_version
 
 
-#: (:class:`collections.Mapping`) The dictionary of output styles.
+#: (:class:`collections.abc.Mapping`) The dictionary of output styles.
 #: Keys are output name strings, and values are flag integers.
 OUTPUT_STYLES = _sass.OUTPUT_STYLES
 
-#: (:class:`collections.Mapping`) The dictionary of source comments styles.
+#: (:class:`collections.abc.Mapping`) The dictionary of source comments styles.
 #: Keys are mode names, and values are corresponding flag integers.
 #:
 #: .. versionadded:: 0.4.0
@@ -47,8 +47,8 @@ OUTPUT_STYLES = _sass.OUTPUT_STYLES
 #: .. deprecated:: 0.6.0
 SOURCE_COMMENTS = {'none': 0, 'line_numbers': 1, 'default': 1, 'map': 2}
 
-#: (:class:`collections.Set`) The set of keywords :func:`compile()` can take.
-MODES = set(['string', 'filename', 'dirname'])
+#: (:class:`frozenset`) The set of keywords :func:`compile()` can take.
+MODES = frozenset(('string', 'filename', 'dirname'))
 
 
 def to_native_s(s):
@@ -83,9 +83,9 @@ class SassFunction(object):
     :param name: the function name
     :type name: :class:`str`
     :param arguments: the argument names
-    :type arguments: :class:`collections.Sequence`
+    :type arguments: :class:`collections.abc.Sequence`
     :param callable_: the actual function to be called
-    :type callable_: :class:`collections.Callable`
+    :type callable_: :class:`collections.abc.Callable`
 
     .. versionadded:: 0.7.0
 
@@ -107,14 +107,22 @@ class SassFunction(object):
         :rtype: :class:`SassFunction`
 
         """
-        argspec = inspect.getargspec(lambda_)
-        if argspec.varargs or argspec.keywords or argspec.defaults:
+        if PY2:  # pragma: no cover
+            a = inspect.getargspec(lambda_)
+            varargs, varkw, defaults, kwonlyargs = (
+                a.varargs, a.keywords, a.defaults, None)
+        else:  # pragma: no cover
+            a = inspect.getfullargspec(lambda_)
+            varargs, varkw, defaults, kwonlyargs = (
+                a.varargs, a.varkw, a.defaults, a.kwonlyargs)
+
+        if varargs or varkw or defaults or kwonlyargs:
             raise TypeError(
                 'functions cannot have starargs or defaults: {0} {1}'.format(
                     name, lambda_
                 )
             )
-        return cls(name, argspec.args, lambda_)
+        return cls(name, a.args, lambda_)
 
     @classmethod
     def from_named_function(cls, function):
@@ -214,7 +222,7 @@ def _raise(e):
 
 def compile_dirname(
     search_path, output_path, output_style, source_comments, include_paths,
-    precision, custom_functions, importers
+    precision, custom_functions, importers, custom_import_extensions
 ):
     fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
     for dirpath, _, filenames in os.walk(search_path, onerror=_raise):
@@ -231,7 +239,8 @@ def compile_dirname(
             input_filename = input_filename.encode(fs_encoding)
             s, v, _ = _sass.compile_filename(
                 input_filename, output_style, source_comments, include_paths,
-                precision, None, custom_functions, importers
+                precision, None, custom_functions, importers, None,
+                custom_import_extensions,
             )
             if s:
                 v = v.decode('UTF-8')
@@ -259,11 +268,11 @@ def compile(**kwargs):
     """There are three modes of parameters :func:`compile()` can take:
     ``string``, ``filename``, and ``dirname``.
 
-    The ``string`` parameter is the most basic way to compile SASS.
-    It simply takes a string of SASS code, and then returns a compiled
+    The ``string`` parameter is the most basic way to compile Sass.
+    It simply takes a string of Sass code, and then returns a compiled
     CSS string.
 
-    :param string: SASS source code to compile.  it's exclusive to
+    :param string: Sass source code to compile.  it's exclusive to
                    ``filename`` and ``dirname`` parameters
     :type string: :class:`str`
     :param output_style: an optional coding style of the compiled result.
@@ -274,32 +283,35 @@ def compile(**kwargs):
                             :const:`False` by default
     :type source_comments: :class:`bool`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
-    :param indented: optional declaration that the string is SASS, not SCSS
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: optional extra file extensions which
+                                     allow can be imported, eg. ``['.css']``
+    :type custom_import_extensions: :class:`list`, :class:`tuple`
+    :param indented: optional declaration that the string is Sass, not SCSS
                      formatted. :const:`False` by default
     :type indented: :class:`bool`
     :returns: the compiled CSS string
     :param importers: optional callback functions.
                      see also below `importer callbacks
-                     <importer-callbacks>`_ description
-    :type importers: :class:`collections.Callable`
+                     <importer-callbacks_>`_ description
+    :type importers: :class:`collections.abc.Callable`
     :rtype: :class:`str`
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
 
     The ``filename`` is the most commonly used way.  It takes a string of
-    SASS filename, and then returns a compiled CSS string.
+    Sass filename, and then returns a compiled CSS string.
 
-    :param filename: the filename of SASS source code to compile.
+    :param filename: the filename of Sass source code to compile.
                      it's exclusive to ``string`` and ``dirname`` parameters
     :type filename: :class:`str`
     :param output_style: an optional coding style of the compiled result.
@@ -314,31 +326,34 @@ def compile(**kwargs):
                                 using source maps.  :const:`None` by default.
     :type source_map_filename: :class:`str`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: optional extra file extensions which
+                                     allow can be imported, eg. ``['.css']``
+    :type custom_import_extensions: :class:`list`, :class:`tuple`
     :param importers: optional callback functions.
                      see also below `importer callbacks
-                     <importer-callbacks>`_ description
-    :type importers: :class:`collections.Callable`
+                     <importer-callbacks_>`_ description
+    :type importers: :class:`collections.abc.Callable`
     :returns: the compiled CSS string, or a pair of the compiled CSS string
               and the source map string if ``source_map_filename`` is set
     :rtype: :class:`str`, :class:`tuple`
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
     :raises exceptions.IOError: when the ``filename`` doesn't exist or
                                 cannot be read
 
     The ``dirname`` is useful for automation.  It takes a pair of paths.
     The first of the ``dirname`` pair refers the source directory, contains
-    several SASS source files to compiled.  SASS source files can be nested
+    several Sass source files to compiled.  Sass source files can be nested
     in directories.  The second of the pair refers the output directory
     that compiled CSS files would be saved.  Directory tree structure of
     the source directory will be maintained in the output directory as well.
@@ -356,24 +371,27 @@ def compile(**kwargs):
                             :const:`False` by default
     :type source_comments: :class:`bool`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: optional extra file extensions which
+                                     allow can be imported, eg. ``['.css']``
+    :type custom_import_extensions: :class:`list`, :class:`tuple`
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
 
     .. _custom-functions:
 
     The ``custom_functions`` parameter can take three types of forms:
 
-    :class:`~collections.Set`/:class:`~collections.Sequence` of \
+    :class:`~set`/:class:`~collections.abc.Sequence` of \
     :class:`SassFunction`\ s
        It is the most general form.  Although pretty verbose, it can take
        any kind of callables like type objects, unnamed functions,
@@ -389,7 +407,7 @@ def compile(**kwargs):
               }
           )
 
-    :class:`~collections.Mapping` of names to functions
+    :class:`~collections.abc.Mapping` of names to functions
        Less general, but easier-to-use form.  Although it's not it can take
        any kind of callables, it can take any kind of *functions* defined
        using :keyword:`def`/:keyword:`lambda` syntax.
@@ -406,7 +424,7 @@ def compile(**kwargs):
               }
           )
 
-    :class:`~collections.Set`/:class:`~collections.Sequence` of \
+    :class:`~set`/:class:`~collections.abc.Sequence` of \
     named functions
        Not general, but the easiest-to-use form for *named* functions.
        It can take only named functions, defined using :keyword:`def`.
@@ -531,30 +549,29 @@ def compile(**kwargs):
         raise TypeError('source_comments must be bool, not ' +
                         repr(source_comments))
     fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
-    source_map_filename = kwargs.pop('source_map_filename', None)
-    if not (source_map_filename is None or
-            isinstance(source_map_filename, string_types)):
-        raise TypeError('source_map_filename must be a string, not ' +
-                        repr(source_map_filename))
-    elif isinstance(source_map_filename, text_type):
-        source_map_filename = source_map_filename.encode(fs_encoding)
-    if not ('filename' in modes or source_map_filename is None):
-        raise CompileError('source_map_filename is only available with '
-                           'filename= keyword argument since it has to be '
-                           'aware of it')
-    try:
-        include_paths = kwargs.pop('include_paths') or b''
-    except KeyError:
-        include_paths = b''
-    else:
-        if isinstance(include_paths, collections.Sequence):
-            include_paths = os.pathsep.join(include_paths)
-        elif not isinstance(include_paths, string_types):
-            raise TypeError('include_paths must be a sequence of strings, or '
-                            'a colon-separated (or semicolon-separated if '
-                            'Windows) string, not ' + repr(include_paths))
-        if isinstance(include_paths, text_type):
-            include_paths = include_paths.encode(fs_encoding)
+
+    def _get_file_arg(key):
+        ret = kwargs.pop(key, None)
+        if ret is not None and not isinstance(ret, string_types):
+            raise TypeError('{} must be a string, not {!r}'.format(key, ret))
+        elif isinstance(ret, text_type):
+            ret = ret.encode(fs_encoding)
+        if ret and 'filename' not in modes:
+            raise CompileError(
+                '{} is only available with filename= keyword argument since '
+                'has to be aware of it'.format(key)
+            )
+        return ret
+
+    source_map_filename = _get_file_arg('source_map_filename')
+    output_filename_hint = _get_file_arg('output_filename_hint')
+
+    # #208: cwd is always included in include paths
+    include_paths = (os.getcwd(),)
+    include_paths += tuple(kwargs.pop('include_paths', ()) or ())
+    include_paths = os.pathsep.join(include_paths)
+    if isinstance(include_paths, text_type):
+        include_paths = include_paths.encode(fs_encoding)
 
     custom_functions = kwargs.pop('custom_functions', ())
     if isinstance(custom_functions, collections.Mapping):
@@ -577,6 +594,14 @@ def compile(**kwargs):
             'not {1!r}'.format(SassFunction, custom_functions)
         )
 
+    _custom_exts = kwargs.pop('custom_import_extensions', []) or []
+    if not isinstance(_custom_exts, (list, tuple)):
+        raise TypeError(
+            'custom_import_extensions must be a list of strings '
+            'not {}'.format(type(_custom_exts))
+        )
+    custom_import_extensions = [ext.encode('utf-8') for ext in _custom_exts]
+
     importers = _validate_importers(kwargs.pop('importers', None))
 
     if 'string' in modes:
@@ -590,7 +615,7 @@ def compile(**kwargs):
         _check_no_remaining_kwargs(compile, kwargs)
         s, v = _sass.compile_string(
             string, output_style, source_comments, include_paths, precision,
-            custom_functions, indented, importers,
+            custom_functions, indented, importers, custom_import_extensions,
         )
         if s:
             return v.decode('utf-8')
@@ -606,6 +631,7 @@ def compile(**kwargs):
         s, v, source_map = _sass.compile_filename(
             filename, output_style, source_comments, include_paths, precision,
             source_map_filename, custom_functions, importers,
+            output_filename_hint, custom_import_extensions,
         )
         if s:
             v = v.decode('utf-8')
@@ -623,6 +649,7 @@ def compile(**kwargs):
         s, v = compile_dirname(
             search_path, output_path, output_style, source_comments,
             include_paths, precision, custom_functions, importers,
+            custom_import_extensions
         )
         if s:
             return
@@ -639,7 +666,7 @@ def and_join(strings):
     'Korea, Japan, China, and Taiwan'
 
     :param strings: a list of words to join
-    :type string: :class:`collections.Sequence`
+    :type string: :class:`collections.abc.Sequence`
     :returns: a joined string
     :rtype: :class:`str`, :class:`basestring`
 
@@ -694,12 +721,15 @@ SASS_SEPARATOR_SPACE = collections.namedtuple('SASS_SEPARATOR_SPACE', ())()
 SEPARATORS = frozenset((SASS_SEPARATOR_COMMA, SASS_SEPARATOR_SPACE))
 
 
-class SassList(collections.namedtuple('SassList', ('items', 'separator'))):
+class SassList(collections.namedtuple(
+        'SassList', ('items', 'separator', 'bracketed'),
+)):
 
-    def __new__(cls, items, separator):
+    def __new__(cls, items, separator, bracketed=False):
         items = tuple(items)
-        assert separator in SEPARATORS
-        return super(SassList, cls).__new__(cls, items, separator)
+        assert separator in SEPARATORS, separator
+        assert isinstance(bracketed, bool), bracketed
+        return super(SassList, cls).__new__(cls, items, separator, bracketed)
 
 
 class SassError(collections.namedtuple('SassError', ('msg',))):
