@@ -17,7 +17,7 @@ from twine.commands import upload
 
 APPVEYOR_API_BASE_URL = 'https://ci.appveyor.com/api/'
 APPVEYOR_API_PROJECT_URL = urljoin(APPVEYOR_API_BASE_URL,
-                                   'projects/dahlia/libsass-python/')
+                                   'projects/asottile/libsass-python/')
 APPVEYOR_API_BUILDS_URL = urljoin(APPVEYOR_API_PROJECT_URL,
                                   'history?recordsNumber=50&branch=master')
 APPVEYOR_API_JOBS_URL = urljoin(APPVEYOR_API_PROJECT_URL,
